@@ -1,62 +1,17 @@
 /*
- * $Source: /home/cvs/jakarta-commons/dbcp/src/java/org/apache/commons/dbcp/PoolableConnectionFactory.java,v $
- * $Revision: 1.15 $
- * $Date: 2003/10/09 21:04:44 $
- *
- * ====================================================================
- *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowledgement:
- *       "This product includes software developed by the
- *        Apache Software Foundation - http://www.apache.org/"
- *    Alternately, this acknowledgement may appear in the software itself,
- *    if and wherever such third-party acknowledgements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * http://www.apache.org/
- *
+ * Copyright 1999-2004 The Apache Software Foundation.
+ * 
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.commons.dbcp;
@@ -73,9 +28,9 @@ import org.apache.commons.pool.*;
  *
  * @author Rodney Waldhoff
  * @author Glenn L. Nielsen
- * @author James House (<a href="mailto:james@interobjective.com">james@interobjective.com</a>)
+ * @author James House
  * @author Dirk Verbeeck
- * @version $Id: PoolableConnectionFactory.java,v 1.15 2003/10/09 21:04:44 rdonkin Exp $
+ * @version $Revision: 1.22 $ $Date: 2004/05/20 13:11:56 $
  */
 public class PoolableConnectionFactory implements PoolableObjectFactory {
     /**
@@ -83,7 +38,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
      * @param connFactory the {@link ConnectionFactory} from which to obtain base {@link Connection}s
      * @param pool the {@link ObjectPool} in which to pool those {@link Connection}s
      * @param stmtPoolFactory the {@link KeyedObjectPoolFactory} to use to create {@link KeyedObjectPool}s for pooling {@link java.sql.PreparedStatement}s, or <tt>null</tt> to disable {@link java.sql.PreparedStatement} pooling
-     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. May be <tt>null</tt>
+     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. Using <tt>null</tt> turns off validation.
      * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
      * @param defaultAutoCommit the default "auto commit" setting for returned {@link Connection}s
      */
@@ -93,7 +48,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         _pool.setFactory(this);
         _stmtPoolFactory = stmtPoolFactory;
         _validationQuery = validationQuery;
-        _defaultReadOnly = defaultReadOnly;
+        _defaultReadOnly = Boolean.valueOf(defaultReadOnly);
         _defaultAutoCommit = defaultAutoCommit;
     }
 
@@ -102,7 +57,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
      * @param connFactory the {@link ConnectionFactory} from which to obtain base {@link Connection}s
      * @param pool the {@link ObjectPool} in which to pool those {@link Connection}s
      * @param stmtPoolFactory the {@link KeyedObjectPoolFactory} to use to create {@link KeyedObjectPool}s for pooling {@link java.sql.PreparedStatement}s, or <tt>null</tt> to disable {@link java.sql.PreparedStatement} pooling
-     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. May be <tt>null</tt>
+     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. Using <tt>null</tt> turns off validation.
      * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
      * @param defaultAutoCommit the default "auto commit" setting for returned {@link Connection}s
      * @param defaultTransactionIsolation the default "Transaction Isolation" setting for returned {@link Connection}s
@@ -113,7 +68,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         _pool.setFactory(this);
         _stmtPoolFactory = stmtPoolFactory;
         _validationQuery = validationQuery;
-        _defaultReadOnly = defaultReadOnly;
+        _defaultReadOnly = Boolean.valueOf(defaultReadOnly);
         _defaultAutoCommit = defaultAutoCommit;
         _defaultTransactionIsolation = defaultTransactionIsolation;
     }
@@ -123,7 +78,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
      * @param connFactory the {@link ConnectionFactory} from which to obtain base {@link Connection}s
      * @param pool the {@link ObjectPool} in which to pool those {@link Connection}s
      * @param stmtPoolFactory the {@link KeyedObjectPoolFactory} to use to create {@link KeyedObjectPool}s for pooling {@link java.sql.PreparedStatement}s, or <tt>null</tt> to disable {@link java.sql.PreparedStatement} pooling
-     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. May be <tt>null</tt>
+     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. Using <tt>null</tt> turns off validation.
      * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
      * @param defaultAutoCommit the default "auto commit" setting for returned {@link Connection}s
      * @param config the AbandonedConfig if tracing SQL objects
@@ -144,7 +99,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         _pool.setFactory(this);
         _stmtPoolFactory = stmtPoolFactory;
         _validationQuery = validationQuery;
-        _defaultReadOnly = defaultReadOnly;
+        _defaultReadOnly = Boolean.valueOf(defaultReadOnly);
         _defaultAutoCommit = defaultAutoCommit;
     }
 
@@ -153,7 +108,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
      * @param connFactory the {@link ConnectionFactory} from which to obtain base {@link Connection}s
      * @param pool the {@link ObjectPool} in which to pool those {@link Connection}s
      * @param stmtPoolFactory the {@link KeyedObjectPoolFactory} to use to create {@link KeyedObjectPool}s for pooling {@link java.sql.PreparedStatement}s, or <tt>null</tt> to disable {@link java.sql.PreparedStatement} pooling
-     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. May be <tt>null</tt>
+     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. Using <tt>null</tt> turns off validation.
      * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
      * @param defaultAutoCommit the default "auto commit" setting for returned {@link Connection}s
      * @param defaultTransactionIsolation the default "Transaction Isolation" setting for returned {@link Connection}s
@@ -176,7 +131,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         _pool.setFactory(this);
         _stmtPoolFactory = stmtPoolFactory;
         _validationQuery = validationQuery;
-        _defaultReadOnly = defaultReadOnly;
+        _defaultReadOnly = Boolean.valueOf(defaultReadOnly);
         _defaultAutoCommit = defaultAutoCommit;
         _defaultTransactionIsolation = defaultTransactionIsolation;
     }
@@ -186,7 +141,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
      * @param connFactory the {@link ConnectionFactory} from which to obtain base {@link Connection}s
      * @param pool the {@link ObjectPool} in which to pool those {@link Connection}s
      * @param stmtPoolFactory the {@link KeyedObjectPoolFactory} to use to create {@link KeyedObjectPool}s for pooling {@link java.sql.PreparedStatement}s, or <tt>null</tt> to disable {@link java.sql.PreparedStatement} pooling
-     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. May be <tt>null</tt>
+     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. Using <tt>null</tt> turns off validation.
      * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
      * @param defaultAutoCommit the default "auto commit" setting for returned {@link Connection}s
      * @param defaultTransactionIsolation the default "Transaction Isolation" setting for returned {@link Connection}s
@@ -205,6 +160,41 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         String defaultCatalog,
         AbandonedConfig config) {
             
+        _connFactory = connFactory;
+        _pool = pool;
+        _config = config;
+        _pool.setFactory(this);
+        _stmtPoolFactory = stmtPoolFactory;
+        _validationQuery = validationQuery;
+        _defaultReadOnly = Boolean.valueOf(defaultReadOnly);
+        _defaultAutoCommit = defaultAutoCommit;
+        _defaultTransactionIsolation = defaultTransactionIsolation;
+        _defaultCatalog = defaultCatalog;
+    }
+
+    /**
+     * Create a new <tt>PoolableConnectionFactory</tt>.
+     * @param connFactory the {@link ConnectionFactory} from which to obtain base {@link Connection}s
+     * @param pool the {@link ObjectPool} in which to pool those {@link Connection}s
+     * @param stmtPoolFactory the {@link KeyedObjectPoolFactory} to use to create {@link KeyedObjectPool}s for pooling {@link java.sql.PreparedStatement}s, or <tt>null</tt> to disable {@link java.sql.PreparedStatement} pooling
+     * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.  Should return at least one row. Using <tt>null</tt> turns off validation.
+     * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
+     * @param defaultAutoCommit the default "auto commit" setting for returned {@link Connection}s
+     * @param defaultTransactionIsolation the default "Transaction Isolation" setting for returned {@link Connection}s
+     * @param defaultCatalog the default "catalog" setting for returned {@link Connection}s
+     * @param config the AbandonedConfig if tracing SQL objects
+     */
+    public PoolableConnectionFactory(
+        ConnectionFactory connFactory,
+        ObjectPool pool,
+        KeyedObjectPoolFactory stmtPoolFactory,
+        String validationQuery,
+        Boolean defaultReadOnly,
+        boolean defaultAutoCommit,
+        int defaultTransactionIsolation,
+        String defaultCatalog,
+        AbandonedConfig config) {
+            
         _connFactory = connFactory;
         _pool = pool;
         _config = config;
@@ -228,7 +218,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
     /**
      * Sets the query I use to {@link #validateObject validate} {@link Connection}s.
      * Should return at least one row.
-     * May be <tt>null</tt>
+     * Using <tt>null</tt> turns off validation.
      * @param validationQuery a query to use to {@link #validateObject validate} {@link Connection}s.
      */
     synchronized public void setValidationQuery(String validationQuery) {
@@ -269,7 +259,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
      * @param defaultReadOnly the default "read only" setting for borrowed {@link Connection}s
      */
     public void setDefaultReadOnly(boolean defaultReadOnly) {
-        _defaultReadOnly = defaultReadOnly;
+        _defaultReadOnly = Boolean.valueOf(defaultReadOnly);
     }
 
     /**
@@ -312,7 +302,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         }
     }
 
-    synchronized public boolean validateObject(Object obj) {
+    public boolean validateObject(Object obj) {
         if(obj instanceof Connection) {
             try {
                 validateConnection((Connection) obj);
@@ -325,7 +315,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         }
     }
 
-    synchronized public void validateConnection(Connection conn) throws SQLException {
+    public void validateConnection(Connection conn) throws SQLException {
         String query = _validationQuery;
         if(conn.isClosed()) {
             throw new SQLException("validateConnection: connection closed");
@@ -358,7 +348,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
     public void passivateObject(Object obj) throws Exception {
         if(obj instanceof Connection) {
             Connection conn = (Connection)obj;
-            if(!conn.getAutoCommit()) {
+            if(!conn.getAutoCommit() && !conn.isReadOnly()) {
                 conn.rollback();
             }
             conn.clearWarnings();
@@ -376,10 +366,12 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         if(obj instanceof Connection) {
             Connection conn = (Connection)obj;
             conn.setAutoCommit(_defaultAutoCommit);
-            conn.setReadOnly(_defaultReadOnly);
             if (_defaultTransactionIsolation != UNKNOWN_TRANSACTIONISOLATION) {
                 conn.setTransactionIsolation(_defaultTransactionIsolation);
             }
+            if (_defaultReadOnly != null) {
+                conn.setReadOnly(_defaultReadOnly.booleanValue());
+            }
             if (_defaultCatalog != null) {
                 conn.setCatalog(_defaultCatalog);
             }
@@ -390,7 +382,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
     protected String _validationQuery = null;
     protected ObjectPool _pool = null;
     protected KeyedObjectPoolFactory _stmtPoolFactory = null;
-    protected boolean _defaultReadOnly = false;
+    protected Boolean _defaultReadOnly = null;
     protected boolean _defaultAutoCommit = true;
     protected int _defaultTransactionIsolation = UNKNOWN_TRANSACTIONISOLATION;
     protected String _defaultCatalog;
